#include <DualWheelTruck.h> // Подключение библиотеки <DualWheelTruck.h> для работы с моторами
#include <SmoothServo.h> // Подключение библиотеки <SmoothServo.h> для работы с сервоприводами
#include <GW_US04.h> // Подключение библиотеки <GW_US04.h> для работы с ультразвуковым дальномером
#include <LiquidCrystal_I2C_OLED.h> // Подключение библиотеки <LiquidCrystal_I2C_OLED.h> для работы с I2C дисплеем

#define SHLAGBAUM_UP 0 // Назначение угла поворота вала сервопривода, при котором шлагбаум поднят
#define SHLAGBAUM_DOWN 50 // Назначение угла поворота вала сервопривода, при котором шлагбаум опущен

#define BUTTON_OK 5 // Назначение пина, к которому подключена кнопка подтверждения выбора
#define BUTTON_NUMBER 11 // Назначение пина, к которому подключена кнопка переключения номера
#define TRIG_PIN 4 // Назначение пина, к которому подключен вывод trig ультразвукового дальномера
#define ECHO_PIN 9 // Назначение пина, к которому подключен вывод echo ультразвукового дальномера

bool button_ok_state=false; // Объявление переменной button_ok_state и присвоение ей значения false
bool button_number_state=false; // Объявление переменной button_number_state и присвоение ей значения false

bool codeStatus = false; // Б.1 - Объявление переменной codeStatus и присвоение ей значения false
bool gateClosed = false; // Б.1 - Объявление переменной gateClosed и присвоение ей значения false

int speeds=100; // Б.1 - Объявление переменной speeds и присвоение ей значения 100
int distance; // Объявление переменной distance

int savedCode[4] = {4, 3, 2, 1}; // Б.1 - Объявление числового массима savedCode и присвоение его элементам значений 4,3,2,1
int enteredCode[4] = {0,0,0,0}; // Б.1 - Объявление числового массима enteredCode
int enteringDigit = 0; // Б.1 - Объявление переменной enteringDigit и присвоение ей значения 0
int enteringNumber = -1; // Б.1 - Объявление переменной enteringNumber и присвоение ей значения -1
int counter; // Объявление переменной counter

void button_read(); // Объявление функции button_read()
void oled_write(); // Объявление функции oled_write()

DCMotor* motor_right; // Инициализация мотора и присвоение ему имени motor_right
DCMotor* motor_left; // Инициализация мотора и присвоение ему имени motor_left

SmoothServo* shlagbaum; // Инициализация сервопривода и присвоение ему имени shlagbaum

GW_US04 sonar = GW_US04(TRIG_PIN, ECHO_PIN); // Инициализация ультразвукового дальномера и присвоение ему имени sonar

LiquidCrystal_I2C OLED1(0x38,16,2);  // Инициализация I2C дисплея: адрес дисплея 0х38, 16 символов в строке, 2 строки

void setup() {
Board::vertor_PCA(); // Объявление типа используемого контроллера - vertor_PCA

motor_right = new DCMotor(1, 0, true, 5); // Настройка мотора motor_right с применением функции DCMotor(a, b, c, d): подключен к клеммнику 1, прямая полярность подключения мотора, инерционность 5
motor_left = new DCMotor(0, 0, false, 5); // Настройка мотора motor_left с применением функции DCMotor(a, b, c, d): подключен к клеммнику 0, обратная полярность подключения мотора, инерционность 5

motor_right->stop(false); // Команда на остановку вращения мотора motor_right
motor_left->stop(false); // Команда на остановку вращения мотора motor_left

shlagbaum = new SmoothServo(15); // Указание номера штыря, к которому подключен сервопривод shlagbaum

shlagbaum->setMinAngularSpeed(70); // Установка минимальной угловой скорости поворота вала сервопривода shlagbaum
shlagbaum->setMaxAngularSpeed(180); // Установка максимальной угловой скорости поворота вала сервопривода shlagbaum

shlagbaum->begin(30); // Команда на запуск работы с сервоприводом shlagbaum

shlagbaum->smoothRotate(SHLAGBAUM_UP); // Б.2 - Команда на плавный поворот вала серводвигателя shlagbaum в положение SHLAGBAUM_UP градусов

OLED1.init(); // Инициализация I2C дислея                     
OLED1.clear(); // Очистка I2C дислея   

pinMode (BUTTON_OK,INPUT); // Настройка пина BUTTON_OK на ввод
pinMode (BUTTON_NUMBER,INPUT); // Настройка пина BUTTON_NUMBER на ввод
}

void loop() {      
 distance=sonar.scan(); // Б.3 - Сохранение значения, возвращаемого функцией sonar.scan() в переменную distance
 delay(30); // Б.4 - Ожидание 0,03 секунды
 if(distance<15) // Б.5 - Если значение переменной distance меньше 15, то начать выполнение инструкций в скобках
 {
   motor_right->forward(speeds); // Б.6 - Команда на запуск мотора motor_right со скоростью speeds % в прямом направлении
   motor_left->forward(speeds); // Б.6 - Команда на запуск мотора motor_left со скоростью speeds % в прямом направлении
   delay(1000); // Б.7 - Ожидание 1 секунда
   motor_right->stop(false); // Б.8 - Команда на остановку вращения мотора motor_right
   motor_left->stop(false); // Б.8 - Команда на остановку вращения мотора motor_left
   delay(500); // Б.9 - Ожидание 0,5 секунды
   shlagbaum->smoothRotate(SHLAGBAUM_DOWN); // Б.10 - Команда на плавный поворот вала серводвигателя shlagbaum в положение SHLAGBAUM_DOWN градусов
   delay(2000); // Б.2 - Ожидание 2 секунды
   OLED1.setCursor(2, 0); // Б.12 - Установка курсора дисплея на позицию 1-я строка, сдвиг 2
   OLED1.outStr("Для прохода"); // Б.12 - Вывод на I2C дисплей сообщения:"Для прохода" 
   OLED1.setCursor(1, 1); // Б.13 - Установка курсора дисплея на позицию 2-я строка, сдвиг 1
   OLED1.outStr("введите пароль!"); // Б.13 - Вывод на I2C дисплей сообщения:"введите пароль"
   delay(1500); // Б.14 - Ожидание 1,5 секунды
   OLED1.clear(); // Б.15 - Очистка I2C дислея  
   while(gateClosed==false) // Б.16 - Если значение переменной gateClosed равно false, то выполнять инструкции в скобках
   {     
     button_read(); // Б.26 - Вызов на исполнение функции button_read()
     oled_write(); // Б.27 - Вызов на исполнение функции oled_write()
     if(enteringDigit==4) // Б.28 - Если значение переменной enteringDigit равно 4, то выполнять инструкции в скобках
     {      
      counter=0; // Б.29 - Присвоение переменной counter значения 0                      
  for (int i = 0; i < 4; i++) // Б.30, Б.31, Б.34 -  - Выполение тела цикла до тех пор, пока значение переменной i меньше 4
  { 
    if (savedCode[i] == enteredCode[i]) // Б.32 - Если значение элемента массива savedCode под номером i равно значению элемента массива enteredCode под номером i, то выполнить инструкцию в скобках
    { 
      counter++; // Б.33 - Увеличени значения переменной counter на 1
    }
  }
  if (counter == 4) // Б.35 - Если значение переменной counter равно 4, то выполнить инструкции в скобках
  {                     
      OLED1.clear(); // Б.44 - Очистка I2C дислея
      OLED1.setCursor(2, 0); // Б.45 - Установка курсора дисплея на позицию 1-я строка, сдвиг 2
      OLED1.outStr("Пароль верен."); // Б.45 - Вывод на I2C дисплей сообщения:"Пароль верен"    
      OLED1.setCursor(3, 1); // Б.46 - Установка курсора дисплея на позицию 2-я строка, сдвиг 3
      OLED1.outStr("Проходите!"); // Б.46 - Вывод на I2C дисплей сообщения:"Проходите!"
      gateClosed=true; // Б.47 - Присвоение переменной gateClosed значения false        
      delay(1500); // Б.48 - Ожидание 1,5 секунды                 
  } 
  else 
  {                          
     OLED1.clear(); // Б.36 - Очистка I2C дислея
     OLED1.setCursor(0, 0); // Б.37 - Установка курсора дисплея на позицию 1-я строка, сдвиг 0
     OLED1.outStr("Неверный пароль!"); // Б.37 - Вывод на I2C дисплей сообщения:"Неверный пароль!"    
     OLED1.setCursor(0, 1); // Б.38 - Установка курсора дисплея на позицию 2-я строка, сдвиг 1
     OLED1.outStr("Попробуйте еще!"); // Б.38 - Вывод на I2C дисплей сообщения:"Попробуйте еще!"
     delay(1500); // Б.39 - Ожидание 1,5 секунды 
     enteringDigit = 0; // Б.40 - Присвоение переменной enteringDigit значения 0              
     enteringNumber = -1; // Б.41 - Присвоение переменной enteringNumber значения -1        
     OLED1.clear(); // Б.42 - Очистка I2C дислея   
                 
  }
  for (int i = 0; i < 4; i++) // Б.43, Б.49, Б.51 - Выполение тела цикла до тех пор, пока значение переменной i меньше 4
   {
    enteredCode[i]=0; // Б.50 - Присвоение значению элемента i массива enteredCode значения 0
   }
   }     
   }
   OLED1.clear(); // Б.17 - Очистка I2C дислея  
   shlagbaum->smoothRotate(SHLAGBAUM_UP); // Б.18 - Команда на плавный поворот вала серводвигателя shlagbaum в положение SHLAGBAUM_UP градусов
   delay(2000); // Б.19 - Ожидание 2 секунды 
   motor_right->backward(speeds); // Б.20 - Команда на запуск мотора motor_right со скоростью speeds % в прямом направлении
   motor_left->backward(speeds); // Б.20 - Команда на запуск мотора motor_left со скоростью speeds % в прямом направлении
   delay(1000); // Б.21 - Ожидание 1 секунда
   motor_right->stop(false); // Б.22 - Команда на остановку вращения мотора motor_right
   motor_left->stop(false); // Б.22 - Команда на остановку вращения мотора motor_left
   enteringDigit = 0; // Б.23 - Присвоение переменной enteringDigit значения 0           
   enteringNumber = -1; // Б.24 - Присвоение переменной enteringNumber значения -1  
   gateClosed=false; // Б.25 - Присвоение переменной gateClosed значения false    
 }  
}

void button_read()
{
  button_ok_state=digitalRead(BUTTON_OK); // Б.26.1 - Сохранение состояния на пине BUTTON_START в переменную button_state
  if(button_ok_state==0) // Б.26.2 - Если значение переменной button_state равно 0, то 
  {
  delay(10); // Б.26.3 - Ожидание 0,01 с
  button_ok_state=digitalRead(BUTTON_OK); // Б.26.4 - Сохранение состояния на пине BUTTON_START в переменную button_state
  if(button_ok_state==0) // Б.26.5 - Если значение переменной button_state равно 0, то 
  {
  do 
  {
   button_ok_state=digitalRead(BUTTON_OK); // Б.26.6 - Сохранение состояния на пине BUTTON_START в переменную button_state
  }
  while(button_ok_state==0); // Б.26.7 - Цикл do..while() выполнять до тех пор, пока значение переменной button_state равно 0  
  enteringDigit=enteringDigit+1; // Б.26.8 - Увеличение значения переменной enteringDigit на 1
  enteringNumber=-1; // Б.26.9 - Присвоение переменной enteringNumber значения -1   
  }
  }

  button_number_state=digitalRead(BUTTON_NUMBER); // Б.26.10 - Сохранение состояния на пине BUTTON_START в переменную button_state
  if(button_number_state==0) // Б.26.11 - Если значение переменной button_state равно 0, то 
  {
  delay(10); // Б.26.12 - Ожидание 0,01 с
  button_number_state=digitalRead(BUTTON_NUMBER); // Б.26.13 - Сохранение состояния на пине BUTTON_START в переменную button_state
  if(button_number_state==0) // Б.26.14 - Если значение переменной button_state равно 0, то 
  {
  do 
  {
   button_number_state=digitalRead(BUTTON_NUMBER); // Б.26.15 - Сохранение состояния на пине BUTTON_START в переменную button_state
  }
  while(button_number_state==0); // Б.26.16 - Цикл do..while() выполнять до тех пор, пока значение переменной button_state равно 0  
  enteringNumber=enteringNumber+1; // Б.26.17 - Увеличение значения переменной enteringNumber на 1
  
  if(enteringNumber==10) // Б.26.18 - Если значение переменной enteringNumber равно 10, то выполнить инструкции в скобках 
  {
   enteringNumber=0; // Б.26.19 - Присвоение переменной enteringNumber значения 0   
  }
  enteredCode[enteringDigit]=enteringNumber; // Б.26.20 - Присвоение элементу массива enteredCode под номером enteringDigit значения переменной enteringNumber
  }
  }  
}  

void oled_write()
{
   OLED1.setCursor(0, 0); // Б.27.1 - Установка курсора дисплея на позицию 1-я строка, сдвиг 0
   OLED1.outStr("Ваш код доступа:"); // Б.27.1 - Вывод на I2C дисплей сообщения:"Ваш код доступа:"
   for(int i=0;i<=enteringDigit;i++) // Б.27.2, Б.27.3 - Выполение тела цикла до тех пор, пока значение переменной i меньше значения переменной enteringDigit
   {
    OLED1.setCursor(i, 1); // Установка курсора дисплея на позицию 2-я строка, сдвиг i
    OLED1.print(enteredCode[i]); // Вывод на I2C дисплей сообщения:Значение переменной под номером i массива enteredCode   
   }
   if(enteringNumber==-1&&enteringDigit!=4) // Если значение переменной enteringNumber равно -1 и значение переменной enteringDigit не равно 4, то выполнить инструкции в скобках
   {
    OLED1.setCursor(enteringDigit, 1); // Установка курсора дисплея на позицию 2-я строка, сдвиг enteringDigit
    OLED1.print("_"); // Вывод на I2C дисплей сообщения:"_"
   }
}
